from tfads_o_builder.feature_types._Base import _Base


class SecurityClassification(_Base):
    def __init__(self):
        super().__init__()
        self.field_name = "Security Classification"
        self.note = 'Always "U"'
        self.default_value = "U"

    def validateValue(self):
        # Assert that the value is "U"
        assert self.value in ["U"], 'The value must be "U".'



from tfads_o_builder.feature_types._Base import _Base


class Longitude(_Base):
    def __init__(self):
        super().__init__()
        self.field_name = "Longitude"
        self.note = "(-)Decimal Degrees (example = 177.30485762)"

    def castValue(self):
        # Cast the value as a float
        return float(self.value)

    def validateValue(self):
        # Assert the value is a longitude
        assert -180 <= self.value <= 180, "The value must be a valid longitude."



from tfads_o_builder.feature_types._Base import _Base


class Latitude(_Base):
    def __init__(self):
        super().__init__()
        self.field_name = "Latitude"
        self.note = "(-)Decimal Degrees (example= -17.50982743)"

    def castValue(self):
        # Cast the value as a float
        return float(self.value)

    def validateValue(self):
        # Assert the value is a latitude
        assert -90 <= self.value <= 90, "The value must be a valid latitude."



class _Base:
    def __init__(self):
        self.format = "Char"
        self.field_name = None
        self.note = None
        self.value = None
        self.default_value = None

    """
    Sets the value of the feature type.
    """

    def setValue(self, value):
        # Set the value
        self.value = value

        # Set the default value using the __setDefaultValue function
        self.__setDefaultValue()

        # Cast the value using the castValue function
        self.castValue()

        # Validate the value using the validateValue function
        self.validateValue()

        # Return self
        return self

    """
    Returns the value of the feature type.
    """

    def getValue(self):
        # Set the default value using the __setDefaultValue function
        self.__setDefaultValue()

        # Validate the value using the validateValue function
        self.validateValue()

        # If the value is None, then return an empty string
        if self.value is None:
            return ""

        # Else return the value
        return self.value

    """
    Sets the default value of the feature type.
    """

    def __setDefaultValue(self):
        # If the value is None, then set the value to an empty string
        if self.value is None:
            self.value = self.default_value

    """
    Casts the value of the feature type.
    """

    def castValue(self):
        return str(self.value)

    """
    Validates the value of the feature type.
    """

    def validateValue(self):
        assert True == True

    """
    Returns the field name of the feature type.
    """

    def getFieldName(self):
        # If field_name is None, then raise an error
        if self.field_name is None:
            raise AttributeError("The field_name must be set.")

        return self.field_name



from datetime import datetime
from tfads_o_builder.feature_types._Base import _Base


class SourceDate(_Base):
    def __init__(self):
        super().__init__()
        self.field_name = "Source Date"
        self.note = '"YYYYMMDD" (example 19951026)'

    def validateValue(self):
        # Assert that self.value is not None
        assert self.value is not None, "The CompilationDate is required."

        # Assert that the length of the value is 8
        assert len(self.value) == 8, "The value must be 8 characters in length."

        # Assert that the value is a valid date
        try:
            datetime.strptime(self.value, "%Y%m%d")
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYYMMDD")



from tfads_o_builder.feature_types._Base import _Base


class SingleMultipleFlag(_Base):
    def __init__(self):
        super().__init__()
        self.field_name = "Single Multiple Flag"
        self.note = '"S" or "M"'
        self.default_value = "S"

    def validateValue(self):
        # Assert that the value is either "S" or "M"
        assert self.value in ["S", "M"], 'The value must be either "S" or "M".'



from tfads_o_builder.feature_types.ActiveDeletedCode import ActiveDeletedCode
from tfads_o_builder.feature_types.CompilationDate import CompilationDate
from tfads_o_builder.feature_types.ControlReleasabilityCode import (
    ControlReleasabilityCode,
)
from tfads_o_builder.feature_types.CoordinatePrecisionFlag import (
    CoordinatePrecisionFlag,
)
from tfads_o_builder.feature_types.CountryCode import CountryCode
from tfads_o_builder.feature_types.DeficiencyCode import DeficiencyCode
from tfads_o_builder.feature_types.DeletingOrganization import DeletingOrganization
from tfads_o_builder.feature_types.DerivingOrganization import DerivingOrganization
from tfads_o_builder.feature_types.FACCCode import FACCCode
from tfads_o_builder.feature_types.FACCName import FACCName
from tfads_o_builder.feature_types.FeatureTypeName import FeatureTypeName
from tfads_o_builder.feature_types.HeightAGLAccuracy import HeightAGLAccuracy
from tfads_o_builder.feature_types.HeightAMSLAccuracy import HeightAMSLAccuracy
from tfads_o_builder.feature_types.HorizontalAccuracy import HorizontalAccuracy
from tfads_o_builder.feature_types.HorizontalDatum import HorizontalDatum
from tfads_o_builder.feature_types.Latitude import Latitude
from tfads_o_builder.feature_types.Lights import Lights
from tfads_o_builder.feature_types.LocationElevation import LocationElevation
from tfads_o_builder.feature_types.LocationElevationAccuracy import (
    LocationElevationAccuracy,
)
from tfads_o_builder.feature_types.Longitude import Longitude
from tfads_o_builder.feature_types.MultipleNumber import MultipleNumber
from tfads_o_builder.feature_types.FeatureTypeCode import FeatureTypeCode
from tfads_o_builder.feature_types.ObstructionHeightAGL import ObstructionHeightAGL
from tfads_o_builder.feature_types.ObstructionHeightAMSL import ObstructionHeightAMSL
from tfads_o_builder.feature_types.OriginalID import OriginalID
from tfads_o_builder.feature_types.OutputRemarks import OutputRemarks
from tfads_o_builder.feature_types.PreviousVOIdentifierCode import (
    PreviousVOIdentifierCode,
)
from tfads_o_builder.feature_types.ProcessCode import ProcessCode
from tfads_o_builder.feature_types.ProvinceCode import ProvinceCode
from tfads_o_builder.feature_types.RevisionDate import RevisionDate
from tfads_o_builder.feature_types.SecurityClassification import SecurityClassification
from tfads_o_builder.feature_types.SingleMultipleFlag import SingleMultipleFlag
from tfads_o_builder.feature_types.SourceDate import SourceDate
from tfads_o_builder.feature_types.SurfaceMaterialCode import SurfaceMaterialCode
from tfads_o_builder.feature_types.TransactionType import TransactionType
from tfads_o_builder.feature_types.UUID import UUID
from tfads_o_builder.feature_types.VOIdentification import VOIdentification
from tfads_o_builder.feature_types.VOSequenceNumber import VOSequenceNumber
from tfads_o_builder.feature_types.ValidationCode import ValidationCode
from tfads_o_builder.feature_types.WACINNR import WACINNR
from tfads_o_builder.feature_types.WACNumber import WACNumber


class Point:
    # tab => class type
    # e.g. 1 => VOSequenceNumber
    tab_type_mapping = {
        1: VOSequenceNumber(),
        2: VOIdentification(),
        3: CountryCode(),
        4: ProvinceCode(),
        5: WACNumber(),
        6: Latitude(),
        7: Longitude(),
        8: FeatureTypeCode(),
        9: FeatureTypeName(),
        10: ObstructionHeightAGL(),
        11: ObstructionHeightAMSL(),
        12: LocationElevation(),
        13: HorizontalDatum(),
        14: HeightAGLAccuracy(),
        15: HeightAMSLAccuracy(),
        16: LocationElevationAccuracy(),
        17: HorizontalAccuracy(),
        18: DerivingOrganization(),
        19: SurfaceMaterialCode(),
        20: SingleMultipleFlag(),
        21: MultipleNumber(),
        22: ProcessCode(),
        23: Lights(),
        24: SourceDate(),
        25: RevisionDate(),
        26: CompilationDate(),
        27: SecurityClassification(),
        28: CoordinatePrecisionFlag(),
        29: ValidationCode(),
        30: TransactionType(),
        31: ControlReleasabilityCode(),
        32: DeletingOrganization(),
        33: ActiveDeletedCode(),
        34: DeficiencyCode(),
        35: PreviousVOIdentifierCode(),
        36: OriginalID(),
        37: WACINNR(),
        38: OutputRemarks(),
        39: FACCCode(),
        40: FACCName(),
        41: UUID(),
    }

    # class type => tab
    # e.g. VOSequenceNumber => 1
    type_tab_mapping = {}

    def __init__(self) -> None:
        # Initialize the self.type_tab_mapping setting it to the reverse of the self.tab_type_mapping but the key being the class name
        self.type_tab_mapping = {
            v.__class__.__name__: k for k, v in self.tab_type_mapping.items()
        }

    def __getattr__(self, called_prefix_fn_name=""):
        def default_function(*args, **kwargs):
            # If called_fn_name does not start with "set_", then return None
            if not called_prefix_fn_name.startswith("set") and not called_prefix_fn_name.startswith("get"):
                raise AttributeError(
                    f"The attribute {called_prefix_fn_name} must start with 'set' or 'get'"
                )

            # Trim the "set_" from the called_fn_name
            called_fn_name = called_prefix_fn_name[3:]

            # Check if called_fn_name is in self.remapped_mappings
            if called_fn_name not in self.type_tab_mapping:
                raise AttributeError(f"The attribute {called_fn_name} does not exist.")

            # Get the class from the self.tab_type_mapping
            class_instance = self.tab_type_mapping[
                self.type_tab_mapping[called_fn_name]
            ]

            if called_prefix_fn_name.startswith("get"):
                # Call the getValue function on the instance of the class
                return class_instance.getValue()
            
            if called_prefix_fn_name.startswith("set"):
            # Set the value on the instance of the class
                class_instance.setValue(*args, **kwargs)

        return default_function

    def getFieldNames(self) -> list:
        # Create a list to return
        return_list = []

        # For each tab in the tab_type_mapping, call the getValue function
        for tab, class_instance in self.tab_type_mapping.items():
            return_list.append(class_instance.getFieldName())

        # Return the list
        return return_list

    def getRowData(self):
        # Create a dictionary to return
        return_dict = {}

        # For each tab in the tab_type_mapping, call the getValue function
        for tab, class_instance in self.tab_type_mapping.items():
            return_dict[class_instance.getFieldName()] = class_instance.getValue()

        # Return the dictionary
        return return_dict
